@@ -34,31 +34,48 @@ import version
 from util import print_error, InvalidPassword
 
 import ecdsa
-import aes
+import pyaes
 
 # Bitcoin network constants
 TESTNET = False
+NOLNET = False
 ADDRTYPE_P2PKH = 0
 ADDRTYPE_P2SH = 5
-XPRV_HEADER = "0488ade4"
-XPUB_HEADER = "0488b21e"
+ADDRTYPE_P2WPKH = 6
+XPRV_HEADER = 0x0488ade4
+XPUB_HEADER = 0x0488b21e
 HEADERS_URL = "https://headers.electrum.org/blockchain_headers"
 
 def set_testnet():
-    global ADDRTYPE_P2PKH, ADDRTYPE_P2SH
+    global ADDRTYPE_P2PKH, ADDRTYPE_P2SH, ADDRTYPE_P2WPKH
     global XPRV_HEADER, XPUB_HEADER
     global TESTNET, HEADERS_URL
     TESTNET = True
     ADDRTYPE_P2PKH = 111
     ADDRTYPE_P2SH = 196
-    XPRV_HEADER = "04358394"
-    XPUB_HEADER = "043587cf"
+    ADDRTYPE_P2WPKH = 3
+    XPRV_HEADER = 0x04358394
+    XPUB_HEADER = 0x043587cf
     HEADERS_URL = "https://headers.electrum.org/testnet_headers"
 
+def set_nolnet():
+    global ADDRTYPE_P2PKH, ADDRTYPE_P2SH, ADDRTYPE_P2WPKH
+    global XPRV_HEADER, XPUB_HEADER
+    global NOLNET, HEADERS_URL
+    NOLNET = True
+    ADDRTYPE_P2PKH = 0
+    ADDRTYPE_P2SH = 5
+    ADDRTYPE_P2WPKH = 6
+    XPRV_HEADER = 0x0488ade4
+    XPUB_HEADER = 0x0488b21e
+    HEADERS_URL = "https://headers.electrum.org/nolnet_headers"
+
+
+
 ################################## transactions
 
 FEE_STEP = 10000
-MAX_FEE_RATE = 100000
+MAX_FEE_RATE = 300000
 FEE_TARGETS = [25, 10, 5, 2]
 
 COINBASE_MATURITY = 100
@@ -69,49 +86,52 @@ TYPE_ADDRESS = 0
 TYPE_PUBKEY  = 1
 TYPE_SCRIPT  = 2
 
-
 # AES encryption
-EncodeAES = lambda secret, s: base64.b64encode(aes.encryptData(secret,s))
-DecodeAES = lambda secret, e: aes.decryptData(secret, base64.b64decode(e))
-
-def strip_PKCS7_padding(s):
-    """return s stripped of PKCS7 padding"""
-    if len(s)%16 or not s:
-        raise ValueError("String of len %d can't be PCKS7-padded" % len(s))
-    numpads = ord(s[-1])
-    if numpads > 16:
-        raise ValueError("String ending with %r can't be PCKS7-padded" % s[-1])
-    if s[-numpads:] != numpads*chr(numpads):
-        raise ValueError("Invalid PKCS7 padding")
-    return s[:-numpads]
-
-# backport padding fix to AES module
-aes.strip_PKCS7_padding = strip_PKCS7_padding
+try:
+    from Cryptodome.Cipher import AES
+except:
+    AES = None
 
 def aes_encrypt_with_iv(key, iv, data):
-    mode = aes.AESModeOfOperation.modeOfOperation["CBC"]
-    key = map(ord, key)
-    iv = map(ord, iv)
-    data = aes.append_PKCS7_padding(data)
-    keysize = len(key)
-    assert keysize in aes.AES.keySize.values(), 'invalid key size: %s' % keysize
-    moo = aes.AESModeOfOperation()
-    (mode, length, ciph) = moo.encrypt(data, mode, key, keysize, iv)
-    return ''.join(map(chr, ciph))
+    if AES:
+        padlen = 16 - (len(data) % 16)
+        if padlen == 0:
+            padlen = 16
+        data += chr(padlen) * padlen
+        e = AES.new(key, AES.MODE_CBC, iv).encrypt(data)
+        return e
+    else:
+        aes_cbc = pyaes.AESModeOfOperationCBC(key, iv=iv)
+        aes = pyaes.Encrypter(aes_cbc)
+        e = aes.feed(data) + aes.feed()  # empty aes.feed() appends pkcs padding
+        return e
 
 def aes_decrypt_with_iv(key, iv, data):
-    mode = aes.AESModeOfOperation.modeOfOperation["CBC"]
-    key = map(ord, key)
-    iv = map(ord, iv)
-    keysize = len(key)
-    assert keysize in aes.AES.keySize.values(), 'invalid key size: %s' % keysize
-    data = map(ord, data)
-    moo = aes.AESModeOfOperation()
-    decr = moo.decrypt(data, None, mode, key, keysize, iv)
-    decr = strip_PKCS7_padding(decr)
-    return decr
+    if AES:
+        cipher = AES.new(key, AES.MODE_CBC, iv)
+        data = cipher.decrypt(data)
+        padlen = ord(data[-1])
+        for i in data[-padlen:]:
+            if ord(i) != padlen:
+                raise InvalidPassword()
+        return data[0:-padlen]
+    else:
+        aes_cbc = pyaes.AESModeOfOperationCBC(key, iv=iv)
+        aes = pyaes.Decrypter(aes_cbc)
+        s = aes.feed(data) + aes.feed()  # empty aes.feed() strips pkcs padding
+        return s
 
+def EncodeAES(secret, s):
+    iv = bytes(os.urandom(16))
+    ct = aes_encrypt_with_iv(secret, iv, s)
+    e = iv + ct
+    return base64.b64encode(e)
 
+def DecodeAES(secret, e):
+    e = bytes(base64.b64decode(e))
+    iv, e = e[:16], e[16:]
+    s = aes_decrypt_with_iv(secret, iv, e)
+    return s
 
 def pw_encode(s, password):
     if password:
@@ -120,7 +140,6 @@ def pw_encode(s, password):
     else:
         return s
 
-
 def pw_decode(s, password):
     if password is not None:
         secret = Hash(password)
@@ -174,7 +193,6 @@ def Hash(x):
     if type(x) is unicode: x=x.encode('utf-8')
     return sha256(sha256(x))
 
-
 hash_encode = lambda x: x[::-1].encode('hex')
 hash_decode = lambda x: x.decode('hex')[::-1]
 hmac_sha_512 = lambda x,y: hmac.new(x, y, hashlib.sha512).digest()
@@ -207,6 +225,8 @@ def seed_type(x):
         return 'old'
     elif is_new_seed(x):
         return 'standard'
+    elif TESTNET and is_new_seed(x, version.SEED_PREFIX_SW):
+        return 'segwit'
     elif is_new_seed(x, version.SEED_PREFIX_2FA):
         return '2fa'
     return ''
@@ -247,11 +267,12 @@ def hash_160(public_key):
     md.update(sha256(public_key))
     return md.digest()
 
-def hash_160_to_bc_address(h160, addrtype):
-    vh160 = chr(addrtype) + h160
-    h = Hash(vh160)
-    addr = vh160 + h[0:4]
-    return base_encode(addr, base=58)
+def hash_160_to_bc_address(h160, addrtype, witness_program_version=1):
+    s = chr(addrtype)
+    if addrtype == ADDRTYPE_P2WPKH:
+        s += chr(witness_program_version) + chr(0)
+    s += h160
+    return base_encode(s+Hash(s)[0:4], base=58)
 
 def bc_address_to_hash_160(addr):
     bytes = base_decode(addr, 25, base=58)
@@ -263,9 +284,14 @@ def hash160_to_p2pkh(h160):
 def hash160_to_p2sh(h160):
     return hash_160_to_bc_address(h160, ADDRTYPE_P2SH)
 
-def public_key_to_bc_address(public_key):
+def public_key_to_p2pkh(public_key):
     return hash160_to_p2pkh(hash_160(public_key))
 
+def public_key_to_p2wpkh(public_key):
+    return hash160_to_bc_address(hash_160(public_key), ADDRTYPE_P2WPKH)
+
+
+
 
 __b58chars = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
 assert len(__b58chars) == 58
@@ -392,7 +418,7 @@ def public_key_from_private_key(sec):
 
 def address_from_private_key(sec):
     public_key = public_key_from_private_key(sec)
-    address = public_key_to_bc_address(public_key.decode('hex'))
+    address = public_key_to_p2pkh(public_key.decode('hex'))
     return address
 
 
@@ -458,7 +484,7 @@ def verify_message(address, sig, message):
         public_key, compressed = pubkey_from_signature(sig, message)
         # check public key using the address
         pubkey = point_to_ser(public_key.pubkey.point, compressed)
-        addr = public_key_to_bc_address(pubkey)
+        addr = public_key_to_p2pkh(pubkey)
         if address != addr:
             raise Exception("Bad signature")
         # check message
@@ -637,34 +663,26 @@ class EC_KEY(object):
 
 
     def decrypt_message(self, encrypted):
-
         encrypted = base64.b64decode(encrypted)
-
         if len(encrypted) < 85:
             raise Exception('invalid ciphertext: length')
-
         magic = encrypted[:4]
         ephemeral_pubkey = encrypted[4:37]
         ciphertext = encrypted[37:-32]
         mac = encrypted[-32:]
-
         if magic != 'BIE1':
             raise Exception('invalid ciphertext: invalid magic bytes')
-
         try:
             ephemeral_pubkey = ser_to_point(ephemeral_pubkey)
         except AssertionError, e:
             raise Exception('invalid ciphertext: invalid ephemeral pubkey')
-
         if not ecdsa.ecdsa.point_is_valid(generator_secp256k1, ephemeral_pubkey.x(), ephemeral_pubkey.y()):
             raise Exception('invalid ciphertext: invalid ephemeral pubkey')
-
         ecdh_key = point_to_ser(ephemeral_pubkey * self.privkey.secret_multiplier)
         key = hashlib.sha512(ecdh_key).digest()
         iv, key_e, key_m = key[0:16], key[16:32], key[32:]
         if mac != hmac.new(key_m, encrypted[:-32], hashlib.sha256).digest():
-            raise Exception('invalid ciphertext: invalid mac')
-
+            raise InvalidPassword()
         return aes_decrypt_with_iv(key_e, iv, ciphertext)
 
 
@@ -727,20 +745,35 @@ def _CKD_pub(cK, c, s):
     return cK_n, c_n
 
 
+def xprv_header(xtype):
+    return ("%08x"%(XPRV_HEADER + xtype)).decode('hex')
+
+def xpub_header(xtype):
+    return ("%08x"%(XPUB_HEADER + xtype)).decode('hex')
+
+def serialize_xprv(xtype, c, k, depth=0, fingerprint=chr(0)*4, child_number=chr(0)*4):
+    xprv = xprv_header(xtype) + chr(depth) + fingerprint + child_number + c + chr(0) + k
+    return EncodeBase58Check(xprv)
+
+def serialize_xpub(xtype, c, cK, depth=0, fingerprint=chr(0)*4, child_number=chr(0)*4):
+    xpub = xpub_header(xtype) + chr(depth) + fingerprint + child_number + c + cK
+    return EncodeBase58Check(xpub)
+
 def deserialize_xkey(xkey, prv):
-    header = XPRV_HEADER if prv else XPUB_HEADER
     xkey = DecodeBase58Check(xkey)
-    assert len(xkey) == 78
+    if len(xkey) != 78:
+        raise BaseException('Invalid length')
     depth = ord(xkey[4])
     fingerprint = xkey[5:9]
     child_number = xkey[9:13]
     c = xkey[13:13+32]
-    if xkey[0:4].encode('hex') == header:
-        n = 33 if prv else 32
-        K_or_k = xkey[13+n:]
-    else:
-        raise BaseException('wrong key')
-    return depth, fingerprint, child_number, c, K_or_k
+    header = XPRV_HEADER if prv else XPUB_HEADER
+    xtype = int('0x' + xkey[0:4].encode('hex'), 16) - header
+    if xtype not in ([0, 1] if TESTNET else [0]):
+        raise BaseException('Invalid header')
+    n = 33 if prv else 32
+    K_or_k = xkey[13+n:]
+    return xtype, depth, fingerprint, child_number, c, K_or_k
 
 def deserialize_xpub(xkey):
     return deserialize_xkey(xkey, False)
@@ -748,36 +781,46 @@ def deserialize_xpub(xkey):
 def deserialize_xprv(xkey):
     return deserialize_xkey(xkey, True)
 
+def is_xpub(text):
+    try:
+        deserialize_xpub(text)
+        return True
+    except:
+        return False
+
+def is_xprv(text):
+    try:
+        deserialize_xprv(text)
+        return True
+    except:
+        return False
+
 
 def xpub_from_xprv(xprv):
-    depth, fingerprint, child_number, c, k = deserialize_xprv(xprv)
+    xtype, depth, fingerprint, child_number, c, k = deserialize_xprv(xprv)
     K, cK = get_pubkeys_from_secret(k)
-    xpub = XPUB_HEADER.decode('hex') + chr(depth) + fingerprint + child_number + c + cK
-    return EncodeBase58Check(xpub)
+    return serialize_xpub(xtype, c, cK, depth, fingerprint, child_number)
 
 
-def bip32_root(seed):
+def bip32_root(seed, xtype):
     I = hmac.new("Bitcoin seed", seed, hashlib.sha512).digest()
     master_k = I[0:32]
     master_c = I[32:]
     K, cK = get_pubkeys_from_secret(master_k)
-    xprv = (XPRV_HEADER + "00" + "00000000" + "00000000").decode("hex") + master_c + chr(0) + master_k
-    xpub = (XPUB_HEADER + "00" + "00000000" + "00000000").decode("hex") + master_c + cK
-    return EncodeBase58Check(xprv), EncodeBase58Check(xpub)
+    xprv = serialize_xprv(xtype, master_c, master_k)
+    xpub = serialize_xpub(xtype, master_c, cK)
+    return xprv, xpub
 
-
-def xpub_from_pubkey(cK):
+def xpub_from_pubkey(xtype, cK):
     assert cK[0] in ['\x02','\x03']
-    master_c = chr(0)*32
-    xpub = (XPUB_HEADER + "00" + "00000000" + "00000000").decode("hex") + master_c + cK
-    return EncodeBase58Check(xpub)
+    return serialize_xpub(xtype, chr(0)*32, cK)
 
 
 def bip32_private_derivation(xprv, branch, sequence):
     assert sequence.startswith(branch)
     if branch == sequence:
         return xprv, xpub_from_xprv(xprv)
-    depth, fingerprint, child_number, c, k = deserialize_xprv(xprv)
+    xtype, depth, fingerprint, child_number, c, k = deserialize_xprv(xprv)
     sequence = sequence[len(branch):]
     for n in sequence.split('/'):
         if n == '': continue
@@ -785,18 +828,17 @@ def bip32_private_derivation(xprv, branch, sequence):
         parent_k = k
         k, c = CKD_priv(k, c, i)
         depth += 1
-
     _, parent_cK = get_pubkeys_from_secret(parent_k)
     fingerprint = hash_160(parent_cK)[0:4]
     child_number = ("%08X"%i).decode('hex')
     K, cK = get_pubkeys_from_secret(k)
-    xprv = XPRV_HEADER.decode('hex') + chr(depth) + fingerprint + child_number + c + chr(0) + k
-    xpub = XPUB_HEADER.decode('hex') + chr(depth) + fingerprint + child_number + c + cK
-    return EncodeBase58Check(xprv), EncodeBase58Check(xpub)
+    xpub = serialize_xpub(xtype, c, cK, depth, fingerprint, child_number)
+    xprv = serialize_xprv(xtype, c, k, depth, fingerprint, child_number)
+    return xprv, xpub
 
 
 def bip32_public_derivation(xpub, branch, sequence):
-    depth, fingerprint, child_number, c, cK = deserialize_xpub(xpub)
+    xtype, depth, fingerprint, child_number, c, cK = deserialize_xpub(xpub)
     assert sequence.startswith(branch)
     sequence = sequence[len(branch):]
     for n in sequence.split('/'):
@@ -807,8 +849,7 @@ def bip32_public_derivation(xpub, branch, sequence):
         depth += 1
     fingerprint = hash_160(parent_cK)[0:4]
     child_number = ("%08X"%i).decode('hex')
-    xpub = XPUB_HEADER.decode('hex') + chr(depth) + fingerprint + child_number + c + cK
-    return EncodeBase58Check(xpub)
+    return serialize_xpub(xtype, c, cK, depth, fingerprint, child_number)
 
 
 def bip32_private_key(sequence, k, chain):
@@ -819,6 +860,6 @@ def bip32_private_key(sequence, k, chain):
 
 def xkeys_from_seed(seed, passphrase, derivation):
     from mnemonic import Mnemonic
-    xprv, xpub = bip32_root(Mnemonic.mnemonic_to_seed(seed, passphrase))
+    xprv, xpub = bip32_root(Mnemonic.mnemonic_to_seed(seed, passphrase), 0)
     xprv, xpub = bip32_private_derivation(xprv, "m/", derivation)
     return xprv, xpub
